/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __draw_ViewI_h__
#define __draw_ViewI_h__

namespace fe
{

/**************************************************************************//**
	@brief A rendering region

	@ingroup draw
*//***************************************************************************/
class FE_DL_EXPORT ViewI: virtual public Component
{
	public:

virtual	void			setWindow(sp<WindowI> spWindowI)					=0;
virtual	sp<WindowI>		window(void) const									=0;

virtual	void			setViewMode(sp<ViewMode> spViewMode)				=0;
virtual	sp<ViewMode>	viewMode(void) const								=0;

virtual	void					setProjection(
									ViewMode::Projection a_projection)		=0;
virtual	ViewMode::Projection	projection(void) const						=0;
virtual	void					use(ViewMode::Projection a_projection)		=0;

						/** @brief Determine 3D location of screen coordinate

							By default, the current ViewMode::Projection
							determines the projection method.

							Window coordinates orginate from bottom left. */
virtual	SpatialVector	unproject(Real winx,Real winy,Real winz,
								ViewMode::Projection a_projection=
								ViewMode::e_current) const					=0;

						/** @brief Project 3D location into screen coordinates

							By default, the current ViewMode::Projection
							determines the projection method.

							Window coordinates orginate from bottom left. */
virtual	SpatialVector	project(const SpatialVector& point,
								ViewMode::Projection a_projection=
								ViewMode::e_current) const					=0;

						/** @brief Estimate object's screen size

							The input is a sphere center and radius. */
virtual	Real			pixelSize(const SpatialVector& center,
								Real radius) const							=0;

						/** @brief Estimate object's world size

							The input is a sphere center and projected
							pixel radius. */
virtual	Real			worldSize(const SpatialVector& center,
								Real pixels) const							=0;

						/**	Return a screen aligned rotation. */
virtual	void			screenAlignment(SpatialTransform &a_rotation)		=0;

						/** Given a screen aligned rotation and world location,
							return the screen projected location and scale.
							Useful for normalizing object scale to
							the screen. */
virtual	void			screenInfo(Real &a_screenScale,
							SpatialVector &a_screenLocation,
							const SpatialTransform &a_rotation,
							const SpatialVector &a_location)				=0;

};

} // namespace

#endif /* __draw_ViewI_h__ */
